import { Box, Button, Dialog, IconButton, Typography, useMediaQuery, useTheme } from "@mui/material";
import { useNavigate } from "react-router-dom";
import { useFormik } from "formik";
import CloseIcon from "@mui/icons-material/Close";
import { enqueueSnackbar } from "notistack";
import { object, ref, string } from "yup";
import { useState } from "react";
import {
    PenaTextField,
    getMessageFromFetchError,
    getAuthToken
} from "@frontend/kitui";
import { setUserId } from "../../stores/user";
import { cardShadow } from "../../utils/theme";
import PasswordInput from "../../components/passwordInput";
import {Logo} from "../../images/logo";
import {register} from "../../api/auth";

interface Values {
    email: string;
    password: string;
    repeatPassword: string;
}

const initialValues: Values = {
    email: "",
    password: "",
    repeatPassword: "",
};


const validationSchema = object({
    email: string()
        .required("Поле обязательно")
        .email("Введите корректный email"),
    password: string()
        .min(8, "Минимум 8 символов")
        .matches(
            /^[.,:;\-_+!&()<>\[\]{}№`@"'~*|#$₽%^=?\d\w]+$/,
            "Некорректные символы",
        )
        .required("Поле обязательно"),
    repeatPassword: string()
        .oneOf([ref("password"), undefined], "Пароли не совпадают")
        .required("Повторите пароль"),
});

export default function SignupDialog() {
    const [isDialogOpen, setIsDialogOpen] = useState<boolean>(true);
    const theme = useTheme();
    const upMd = useMediaQuery(theme.breakpoints.up("md"));
    const navigate = useNavigate();
    const token = getAuthToken()
    const formik = useFormik<Values>({
        initialValues,
        validationSchema,
        onSubmit: async (values, formikHelpers) => {
            const [registerResponse, registerError] = await register(
                values.email.trim(),
                values.password.trim(),
                "+7",
            );
            formikHelpers.setSubmitting(false);

            if (registerError) {
                return enqueueSnackbar(registerError);
            }

            if (registerResponse) {
                setUserId(registerResponse._id);
                var link = document.createElement("a");
                link.href =
                    `https://quiz.pena.digital?action=${token}&userId=${registerResponse._id}`;
                document.body.appendChild(link);
                link.click();
            }
        },
    });

    function handleClose() {
        setIsDialogOpen(false);
        setTimeout(() => navigate("/"), theme.transitions.duration.leavingScreen);
    }

    return (
        <Dialog
            open={isDialogOpen}
            onClose={handleClose}
            PaperProps={{
                sx: {
                    width: "600px",
                    maxWidth: "600px",
                },
            }}
            slotProps={{
                backdrop: {
                    style: {
                        backgroundColor: "rgb(0 0 0 / 0.7)",
                    },
                },
            }}
        >
            <Box
                component="form"
                onSubmit={formik.handleSubmit}
                noValidate
                sx={{
                    position: "relative",
                    backgroundColor: "white",
                    display: "flex",
                    alignItems: "center",
                    flexDirection: "column",
                    p: upMd ? "50px" : "18px",
                    pb: upMd ? "40px" : "30px",
                    gap: "15px",
                    borderRadius: "12px",
                    boxShadow: cardShadow,
                }}
            >
                <IconButton
                    onClick={handleClose}
                    sx={{
                        position: "absolute",
                        right: "7px",
                        top: "7px",
                    }}
                >
                    <CloseIcon sx={{ transform: "scale(1.5)" }} />
                </IconButton>
                <Box sx={{ mt: upMd ? undefined : "62px" }}>
                    <Logo width={upMd ? 233 : 196} color="black" />
                </Box>
                <Typography
                    sx={{
                        color: theme.palette.gray.dark,
                        mt: "5px",
                        mb: upMd ? "35px" : "33px",
                    }}
                >
                    Регистрация
                </Typography>
                <PenaTextField
                    value={formik.values.email}
                    placeholder={"username"}
                    onBlur={formik.handleBlur}
                    error={formik.touched.email && Boolean(formik.errors.email)}
                    helperText={formik.touched.email && formik.errors.email}
                    onChange={formik.handleChange}
                    backgroundColor="#F2F3F7"
                    id="email"
                    label="Email"
                    gap={upMd ? "10px" : "10px"}
                />
                <PasswordInput
                    TextfieldProps={{
                        value: formik.values.password,
                        placeholder: "Не менее 8 символов",
                        onBlur: formik.handleBlur,
                        error: formik.touched.password && Boolean(formik.errors.password),
                        helperText: formik.touched.password && formik.errors.password,
                        autoComplete: "new-password",
                    }}
                    onChange={formik.handleChange}
                    color="#F2F3F7"
                    id="password"
                    label="Пароль"
                    gap={upMd ? "10px" : "10px"}
                />
                <PasswordInput
                    TextfieldProps={{
                        value: formik.values.repeatPassword,
                        placeholder: "Не менее 8 символов",
                        onBlur: formik.handleBlur,
                        error: formik.touched.repeatPassword && Boolean(formik.errors.repeatPassword),
                        helperText: formik.touched.repeatPassword && formik.errors.repeatPassword,
                        autoComplete: "new-password",
                    }}
                    onChange={formik.handleChange}
                    color="#F2F3F7"
                    id="repeatPassword"
                    label="Повторить пароль"
                    gap={upMd ? "10px" : "10px"}
                />
                <Button
                    variant="pena-contained-dark"
                    fullWidth
                    type="submit"
                    disabled={formik.isSubmitting}
                    sx={{
                        py: "12px",
                        "&:hover": {
                            backgroundColor: theme.palette.purple.dark,
                        },
                        "&:active": {
                            color: "white",
                            backgroundColor: "black",
                        },
                    }}
                >Зарегистрироваться</Button>
            </Box>
        </Dialog>
    );
}
